@@ -1,12 +1,14 @@
-# ABSTRACT: Subroutines for making simple command line scripts
 package CLI::Helpers;
+# ABSTRACT: Subroutines for making simple command line scripts
+# RECOMMEND PREREQ: App::Nopaste
 
 use strict;
 use warnings;
 
+use Capture::Tiny qw(capture);
 use File::Basename;
 use Getopt::Long qw(:config pass_through);
-use IPC::Run3;
+use Module::Load qw(load);
 use Ref::Util qw(is_ref is_arrayref is_hashref);
 use Sys::Syslog qw(:standard);
 use Term::ANSIColor 2.01 qw(color colored colorstrip);
@@ -14,10 +16,10 @@ use Term::ReadKey;
 use Term::ReadLine;
 use YAML;
 
-our $VERSION = '1.3'; # VERSION
+our $VERSION = '1.4'; # VERSION
 
 our $_OPTIONS_PARSED;
-
+my  @ORIG_ARGS = @ARGV;
 
 { # Work-around for CPAN Smoke Test Failure
     # Details: http://perldoc.perl.org/5.8.9/Term/ReadLine.html#CAVEATS
@@ -53,6 +55,9 @@ if( !defined $_OPTIONS_PARSED ) {
         'syslog-facility:s',
         'syslog-tag:s',
         'syslog-debug!',
+        'tags:s',
+        'nopaste',
+        'nopaste-service:s@',
     );
     $_OPTIONS_PARSED = 1;
 }
@@ -79,6 +84,9 @@ my %DEF = (
     SYSLOG_TAG      => exists $opt{'syslog-tag'}      && length $opt{'syslog-tag'}      ? $opt{'syslog-tag'} : basename($0),
     SYSLOG_FACILITY => exists $opt{'syslog-facility'} && length $opt{'syslog-facility'} ? $opt{'syslog-facility'} : 'local0',
     SYSLOG_DEBUG    => $opt{'syslog-debug'}  || 0,
+    TAGS            => $opt{tags} ? { map { $_ => 1 } split /,/, $opt{tags} } : undef,
+    NOPASTE         => $opt{nopaste} || 0,
+    NOPASTE_SERVICE => $opt{'nopaste-service'} || [ "Shadowcat" ],
 );
 debug({color=>'magenta'}, "CLI::Helpers Definitions");
 debug_var(\%DEF);
@@ -98,14 +106,51 @@ if( $DEF{SYSLOG} ) {
 
 my $TERM = undef;
 my @STICKY = ();
+my @NOPASTE = ();
+my %TAGS    = ();
+
+if( $DEF{NOPASTE} ) {
+    eval {
+        load 'App::Nopaste';
+        1;
+    } or do {
+        $DEF{NOPASTE} = 0;
+        output({stderr=>1,color=>'red',sticky=>1},
+            'App::Nopaste is not installed, please cpanm App::Nopaste for --nopaste support',
+        );
+    };
+}
 
 # Allow some messages to be fired at the end the of program
 END {
+    # Show discovered tags
+    if( keys %TAGS ) {
+        output({color=>'cyan',stderr=>1},
+            sprintf "# Tags discovered: %s",
+                join(', ', map { "$_=$TAGS{$_}" } sort keys %TAGS)
+        );
+    }
+    # Show Sticky Output
     if(@STICKY) {
         foreach my $args (@STICKY) {
             output(@{ $args });
         }
     }
+    # Do the Nopaste
+    if( @NOPASTE ) {
+        my $command_string = join(" ", $0, @ORIG_ARGS);
+        unshift @NOPASTE, "\$ $command_string";
+        my %paste = (
+            text => join("\n", @NOPASTE),
+            summary => $command_string,
+            desc    => $command_string,
+            services => $DEF{NOPASTE_SERVICE},
+        );
+        debug_var(\%paste);
+        output({color=>'cyan',stderr=>1}, "# NoPaste: "
+            . App::Nopaste->nopaste(%paste)
+        );
+    }
     closelog() if $DEF{SYSLOG};
 }
 
@@ -115,18 +160,15 @@ sub def { return exists $DEF{$_[0]} ? $DEF{$_[0]} : undef }
 
 sub git_color_check {
     my @cmd = qw(git config --global --get color.ui);
-    my($out,$err);
-    eval {
-        run3(\@cmd, undef, \$out, \$err);
+    my($stdout,$stderr,$rc) = capture {
+        system @cmd;
     };
-    if( $@  || $err ) {
-        my $error = defined $err ? $err : '';
-        $error   .= defined $@   ? " $@" : '';
-        debug("git_color_check error: $error");
+    if( $rc != 0 ) {
+        debug("git_color_check error: $stderr");
         return 0;
     }
-    debug("git_color_check out: $out");
-    if( $out =~ /auto/ || $out =~ /true/ ) {
+    debug("git_color_check out: $stdout");
+    if( $stdout =~ /auto/ || $stdout =~ /true/ ) {
         return 1;
     }
     return 0;
@@ -175,6 +217,13 @@ sub output {
         @output = map { defined $color ? colorize($color, $_) : $_; } @input;
     }
 
+    # If tagged, we only output if the tag is requested
+    if( $DEF{TAGS} && exists $opts->{tag} ) {
+        # Skip this altogether
+        $TAGS{$opts->{tag}} ||= 0;
+        $TAGS{$opts->{tag}}++;
+        return unless $DEF{TAGS}->{$opts->{tag}};
+    }
     # Out to the console
     if( !$DEF{QUIET} || (exists $opts->{IMPORTANT} && $opts->{IMPORTANT})) {
         my $out_handle = exists $opts->{stderr} && $opts->{stderr} ? \*STDERR : \*STDOUT;
@@ -220,6 +269,9 @@ sub output {
         $o{no_syslog} = 1;
         push @STICKY, [ \%o, @input ];
     }
+    if( $DEF{NOPASTE} ) {
+        push @NOPASTE, @input;
+    }
 }
 
 
@@ -446,7 +498,7 @@ CLI::Helpers - Subroutines for making simple command line scripts
 
 =head1 VERSION
 
-version 1.3
+version 1.4
 
 =head1 SYNOPSIS
 
@@ -514,12 +566,12 @@ Colors would be automatically enabled based on the user's ~/.gitconfig
 
 =head1 OVERVIEW
 
-This module provides a libray of useful functions for constructing simple command
+This module provides a library of useful functions for constructing simple command
 line interfaces.  It is able to extract information from the environment and your
 ~/.gitconfig to display data in a reasonable manner.
 
 Using this module adds argument parsing using L<Getopt::Long> to your script.  It
-enables passthrough, so you can still use your own argument parsing routines or
+enables pass-through, so you can still use your own argument parsing routines or
 Getopt::Long in your script.
 
 =head1 FUNCTIONS
@@ -546,7 +598,7 @@ messages to be output.
 =head2 verbose( \%opts, @messages )
 
 Exported.  Takes an optional hash reference of formatting options.  Automatically
-overrides the B<level> paramter to 1 if it's not set.
+overrides the B<level> parameter to 1 if it's not set.
 
 =head2 debug( \%opts, @messages )
 
@@ -574,7 +626,7 @@ Returns 1 for 'yes' and 0 for 'no'
 
 Exported.  Provides a prompt to the user for input.  If validate is passed, it should be a hash reference
 containing keys of error messages and values which are subroutines to validate the input available as $_.
-If a validator fails, it's error message will be displayed, and the user will be reprompted.
+If a validator fails, it's error message will be displayed, and the user will be re-prompted.
 
 Valid options are:
 
@@ -612,7 +664,7 @@ Exported.  Synonym for text_input("Password: ", noecho => 1);  Also requires the
 
 =head2 prompt("Prompt", options )
 
-Exported.  Wrapper function with rudimentary mimickery of IO::Prompt(er).
+Exported.  Wrapper function with rudimentary mimicry of IO::Prompt(er).
 Uses:
 
     # Mapping back to confirm();
@@ -649,7 +701,7 @@ This module uses L<Sub::Exporter> for flexible imports, the defaults provided by
     text_input ( "Question", validate => { "error message" => sub { length $_[0] } } )
     confirm    ( "Question" )
 
-    prompt()    Wrapper which mimicks IO::Prompt a bit
+    prompt()    Wrapper which mimics IO::Prompt a bit
     pwprompt()  Wrapper to get sensitive data
 
 =head1 ARGS
@@ -657,6 +709,7 @@ This module uses L<Sub::Exporter> for flexible imports, the defaults provided by
 From CLI::Helpers:
 
     --data-file         Path to a file to write lines tagged with 'data => 1'
+    --tags              A comma separated list of tags to display
     --color             Boolean, enable/disable color, default use git settings
     --verbose           Incremental, increase verbosity (Alias is -v)
     --debug             Show developer output
@@ -666,6 +719,18 @@ From CLI::Helpers:
     --syslog-facility   Default "local0"
     --syslog-tag        The program name, default is the script name
     --syslog-debug      Enable debug messages to syslog if in use, default false
+    --nopaste           Use App::Nopaste to paste output to configured paste service
+    --nopaste-service   Named App::Nopaste service to target, defaults to Shadowcat
+
+=head1 NOPASTE
+
+This is optional and will only work if you have L<App::Nopaste> installed.  If
+you just specify C<--nopaste>, any output that would be displayed to the screen
+is submitted to the L<App::Nopaste::Service::Shadowcat> paste bin.  This
+paste service is pretty simple, but works reliably.
+
+During the C<END> block, the output is submitted and the URL of the paste is
+returned to the user.
 
 =head1 OUTPUT OPTIONS
 
@@ -674,6 +739,37 @@ that output.  The hash may contain the following options:
 
 =over 4
 
+=item B<tag>
+
+Add a keyword to tag output with.  The user may then specify C<--tags
+keyword1,keyword2> to only view output at the appropriate level.  This option
+will affect C<data-file> and C<syslog> output.  The output filter requires both
+the presence of the C<tag> in the output options B<and> the user to specify
+C<--tags> on the command line.
+
+Consider a script, C<status.pl>:
+
+    output("System Status: Normal")
+    output({tag=>'foo'}, "Component Foo: OK");
+    output({tag=>'bar'}, "Component Bar: OK");
+
+If an operator runs:
+
+    $ status.pl
+    System Status: Normal
+    Component Foo: OK
+    Component Bar: OK
+
+    $ status.pl --tags bar
+    System Status: Normal
+    Component Bar: OK
+
+    $ status.pl --tags foo
+    System Status: Normal
+    Component Foo: OK
+
+This could be helpful for selecting one or more pertinent tags to display.
+
 =item B<sticky>
 
 Any lines tagged with 'sticky' will be replayed at the end program's end.  This
@@ -681,7 +777,7 @@ is to allow a developer to ensure message are seen at the termination of the pro
 
 =item B<color>
 
-String. Using Term::ANSIColor for output, use the color designated, ie:
+String. Using Term::ANSIColor for output, use the color designated, i.e.:
 
     red,blue,green,yellow,cyan,magenta,white,black, etc..
 
@@ -702,7 +798,7 @@ alert, emerg.
 
 =item B<IMPORTANT>
 
-Bool. Even if --quiet is specified, output this message.  Use sparringly, and yes,
+Bool. Even if --quiet is specified, output this message.  Use sparingly, and yes,
 it is case sensitive.  You need to yell at it for it to yell at your users.
 
 =item B<stderr>
@@ -721,7 +817,7 @@ Integer.  The number of newlines before this output.
 
 =item B<kv>
 
-Bool.  The array of messsages is actually a key/value pair, this implements special coloring and
+Bool.  The array of messages is actually a key/value pair, this implements special coloring and
 expects the number of messages to be even.
 
     output(qw(a 1 b 2));
@@ -751,7 +847,7 @@ Brad Lhotsky <brad@divisionbyzero.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2017 by Brad Lhotsky.
+This software is Copyright (c) 2019 by Brad Lhotsky.
 
 This is free software, licensed under:
 
@@ -794,7 +890,7 @@ MetaCPAN
 
 A modern, open-source CPAN search engine, useful to view POD in HTML format.
 
-L<http://metacpan.org/release/CLI-Helpers>
+L<https://metacpan.org/release/CLI-Helpers>
 
 =item *
 
